"""Test CoolWidget."""

import unittest
from datetime import datetime as datetime_, timedelta
from PySide2 import QtWidgets, QtTest, QtCore, QtGui
from soitool import main

# references:
# * findChild: https://srinikom.github.io/pyside-docs/PySide/QtCore/QObject.html#PySide.QtCore.PySide.QtCore.QObject.findChild
#   * pyside: https://doc.qt.io/qtforpython/PySide2/QtCore/QObject.html#PySide2.QtCore.PySide2.QtCore.QObject.findChild
#   * allows to find child of widget based on type (and name)
#   * findChild in action (C++): https://code.woboq.org/qt5/qtbase/tests/auto/widgets/dialogs/qinputdialog/tst_qinputdialog.cpp.html
#       * look for QInputDialog and QLineEdit
# * thread discussing singleShot: https://github.com/pytest-dev/pytest-qt/issues/256
# * activeModalWidget: https://stackoverflow.com/questions/38596785/how-can-i-get-access-to-a-qmessagebox-by-qtest
# * testing modal dialogs: https://www.qtcentre.org/threads/31239-Testing-modal-dialogs-with-QTestLib
# * keyclick: https://programtalk.com/python-examples/PyQt4.QtTest.QTest.keyClick/
# * isVisible: https://stackoverflow.com/questions/13850240/pyqt-how-to-check-is-qdialog-is-visible

if isinstance(QtGui.qApp, type(None)):
    app = QtWidgets.QApplication([])
else:
    app = QtGui.qApp


def wait(msec):
    """Venter msec millisekunder.

    Bruker https://stackoverflow.com/a/34745326/3545896

    Parameter
    -----
    msec : number
        msec to wait
    """
    end = datetime_.now() + timedelta(milliseconds=msec)
    while datetime_.now() < end:
        app.processEvents()


class TestMain(unittest.TestCase):
    """TestCase for main."""

    def setUp(self):
        """Forbereder widget for testing."""
        self.test_text1 = "A bad boy"
        self.test_text2 = "Hei Anders!"
        self.widget = main.CoolWidget(self.test_text1)
        self.widget.show()

    def test_starts_up(self):
        """Test at widget kan starte opp."""
        self.assertEqual(
            self.widget.qlabel.text(),
            self.test_text1,
        )
        self.assertTrue(self.widget.isVisible())

    def test_change_text_ok(self):
        """Test at endring av tekst funker."""

        def change_text_and_ok():
            # in PySide2 we need to store a reference to this. If we don't the
            # widget is garbage collected somehow before we get to use
            # child_line_edit (a child of the active widget)
            active_widget = app.activeModalWidget()

            child_line_edit = active_widget.findChild(
                QtWidgets.QLineEdit
            )

            QtTest.QTest.keyClicks(child_line_edit, self.test_text2)
            QtTest.QTest.keyClick(child_line_edit, QtCore.Qt.Key_Enter)

        QtCore.QTimer.singleShot(0, change_text_and_ok)
        QtTest.QTest.mouseClick(self.widget.button, QtCore.Qt.LeftButton)

        self.assertEqual(
            self.widget.qlabel.text(),
            self.test_text2,
        )

    def test_change_text_not_ok(self):
        """Test at avbrytelse av endring av tekster ikke endrer tekst."""

        def accept_popup():
            QtTest.QTest.keyClick(app.activeModalWidget(), QtCore.Qt.Key_Enter)

        def change_text_and_not_ok():
            # in PySide2 we need to store a reference to this. If we don't the
            # widget is garbage collected somehow before we get to use
            # child_line_edit (a child of the active widget)
            active_widget = app.activeModalWidget()

            child_line_edit = active_widget.findChild(QtWidgets.QLineEdit)
            QtTest.QTest.keyClicks(child_line_edit, self.test_text2)
            QtCore.QTimer.singleShot(0, accept_popup)
            QtTest.QTest.keyClick(child_line_edit, QtCore.Qt.Key_Escape)

        QtCore.QTimer.singleShot(0, change_text_and_not_ok)

        QtTest.QTest.mouseClick(self.widget.button, QtCore.Qt.LeftButton)

        self.assertNotEqual(
            self.widget.qlabel.text(),
            self.test_text2,
        )


if __name__ == '__main__':
    unittest.main()
